# @generated by generate_proto_mypy_stubs.py.  Do not edit!
import sys
from google.protobuf.descriptor import (
    Descriptor as google___protobuf___descriptor___Descriptor,
    FileDescriptor as google___protobuf___descriptor___FileDescriptor,
)

from google.protobuf.internal.containers import (
    RepeatedCompositeFieldContainer as google___protobuf___internal___containers___RepeatedCompositeFieldContainer,
    RepeatedScalarFieldContainer as google___protobuf___internal___containers___RepeatedScalarFieldContainer,
)

from google.protobuf.message import (
    Message as google___protobuf___message___Message,
)

from typing import (
    Iterable as typing___Iterable,
    Optional as typing___Optional,
    Text as typing___Text,
)

from typing_extensions import (
    Literal as typing_extensions___Literal,
)


builtin___bool = bool
builtin___bytes = bytes
builtin___float = float
builtin___int = int


DESCRIPTOR: google___protobuf___descriptor___FileDescriptor = ...

class Request(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    class Empty(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

        def __init__(self,
            ) -> None: ...
    type___Empty = Empty

    class ScreenshotOptions(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        path: typing___Text = ...
        selector: typing___Text = ...
        fullPage: builtin___bool = ...

        def __init__(self,
            *,
            path : typing___Optional[typing___Text] = None,
            selector : typing___Optional[typing___Text] = None,
            fullPage : typing___Optional[builtin___bool] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"fullPage",b"fullPage",u"path",b"path",u"selector",b"selector"]) -> None: ...
    type___ScreenshotOptions = ScreenshotOptions

    class FilePath(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        path: typing___Text = ...

        def __init__(self,
            *,
            path : typing___Optional[typing___Text] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"path",b"path"]) -> None: ...
    type___FilePath = FilePath

    class Json(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        body: typing___Text = ...

        def __init__(self,
            *,
            body : typing___Optional[typing___Text] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"body",b"body"]) -> None: ...
    type___Json = Json

    class MouseButtonOptions(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        action: typing___Text = ...
        json: typing___Text = ...

        def __init__(self,
            *,
            action : typing___Optional[typing___Text] = None,
            json : typing___Optional[typing___Text] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"action",b"action",u"json",b"json"]) -> None: ...
    type___MouseButtonOptions = MouseButtonOptions

    class KeyboardKeypress(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        action: typing___Text = ...
        key: typing___Text = ...

        def __init__(self,
            *,
            action : typing___Optional[typing___Text] = None,
            key : typing___Optional[typing___Text] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"action",b"action",u"key",b"key"]) -> None: ...
    type___KeyboardKeypress = KeyboardKeypress

    class KeyboardInputOptions(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        action: typing___Text = ...
        input: typing___Text = ...
        delay: builtin___int = ...

        def __init__(self,
            *,
            action : typing___Optional[typing___Text] = None,
            input : typing___Optional[typing___Text] = None,
            delay : typing___Optional[builtin___int] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"action",b"action",u"delay",b"delay",u"input",b"input"]) -> None: ...
    type___KeyboardInputOptions = KeyboardInputOptions

    class Browser(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        browser: typing___Text = ...
        headless: builtin___bool = ...
        rawOptions: typing___Text = ...

        def __init__(self,
            *,
            browser : typing___Optional[typing___Text] = None,
            headless : typing___Optional[builtin___bool] = None,
            rawOptions : typing___Optional[typing___Text] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"browser",b"browser",u"headless",b"headless",u"rawOptions",b"rawOptions"]) -> None: ...
    type___Browser = Browser

    class Context(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        rawOptions: typing___Text = ...
        hideRfBrowser: builtin___bool = ...

        def __init__(self,
            *,
            rawOptions : typing___Optional[typing___Text] = None,
            hideRfBrowser : typing___Optional[builtin___bool] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"hideRfBrowser",b"hideRfBrowser",u"rawOptions",b"rawOptions"]) -> None: ...
    type___Context = Context

    class Url(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        url: typing___Text = ...

        def __init__(self,
            *,
            url : typing___Optional[typing___Text] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"url",b"url"]) -> None: ...
    type___Url = Url

    class TextInput(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        input: typing___Text = ...
        selector: typing___Text = ...
        type: builtin___bool = ...

        def __init__(self,
            *,
            input : typing___Optional[typing___Text] = None,
            selector : typing___Optional[typing___Text] = None,
            type : typing___Optional[builtin___bool] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"input",b"input",u"selector",b"selector",u"type",b"type"]) -> None: ...
    type___TextInput = TextInput

    class ElementProperty(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        selector: typing___Text = ...
        property: typing___Text = ...

        def __init__(self,
            *,
            selector : typing___Optional[typing___Text] = None,
            property : typing___Optional[typing___Text] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"property",b"property",u"selector",b"selector"]) -> None: ...
    type___ElementProperty = ElementProperty

    class TypeText(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        selector: typing___Text = ...
        text: typing___Text = ...
        delay: builtin___int = ...
        clear: builtin___bool = ...

        def __init__(self,
            *,
            selector : typing___Optional[typing___Text] = None,
            text : typing___Optional[typing___Text] = None,
            delay : typing___Optional[builtin___int] = None,
            clear : typing___Optional[builtin___bool] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"clear",b"clear",u"delay",b"delay",u"selector",b"selector",u"text",b"text"]) -> None: ...
    type___TypeText = TypeText

    class FillText(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        selector: typing___Text = ...
        text: typing___Text = ...

        def __init__(self,
            *,
            selector : typing___Optional[typing___Text] = None,
            text : typing___Optional[typing___Text] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"selector",b"selector",u"text",b"text"]) -> None: ...
    type___FillText = FillText

    class ClearText(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        selector: typing___Text = ...

        def __init__(self,
            *,
            selector : typing___Optional[typing___Text] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"selector",b"selector"]) -> None: ...
    type___ClearText = ClearText

    class PressKeys(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        selector: typing___Text = ...
        key: google___protobuf___internal___containers___RepeatedScalarFieldContainer[typing___Text] = ...

        def __init__(self,
            *,
            selector : typing___Optional[typing___Text] = None,
            key : typing___Optional[typing___Iterable[typing___Text]] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"key",b"key",u"selector",b"selector"]) -> None: ...
    type___PressKeys = PressKeys

    class ElementSelector(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        selector: typing___Text = ...

        def __init__(self,
            *,
            selector : typing___Optional[typing___Text] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"selector",b"selector"]) -> None: ...
    type___ElementSelector = ElementSelector

    class Timeout(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        timeout: builtin___float = ...

        def __init__(self,
            *,
            timeout : typing___Optional[builtin___float] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"timeout",b"timeout"]) -> None: ...
    type___Timeout = Timeout

    class Index(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        index: typing___Text = ...

        def __init__(self,
            *,
            index : typing___Optional[typing___Text] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"index",b"index"]) -> None: ...
    type___Index = Index

    class IdWithTimeout(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        id: typing___Text = ...
        timeout: builtin___float = ...

        def __init__(self,
            *,
            id : typing___Optional[typing___Text] = None,
            timeout : typing___Optional[builtin___float] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"id",b"id",u"timeout",b"timeout"]) -> None: ...
    type___IdWithTimeout = IdWithTimeout

    class StyleTag(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        content: typing___Text = ...

        def __init__(self,
            *,
            content : typing___Optional[typing___Text] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"content",b"content"]) -> None: ...
    type___StyleTag = StyleTag

    class ElementSelectorWithOptions(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        selector: typing___Text = ...
        options: typing___Text = ...

        def __init__(self,
            *,
            selector : typing___Optional[typing___Text] = None,
            options : typing___Optional[typing___Text] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"options",b"options",u"selector",b"selector"]) -> None: ...
    type___ElementSelectorWithOptions = ElementSelectorWithOptions

    class ElementSelectorWithDuration(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        selector: typing___Text = ...
        duration: builtin___int = ...
        width: typing___Text = ...
        style: typing___Text = ...
        color: typing___Text = ...

        def __init__(self,
            *,
            selector : typing___Optional[typing___Text] = None,
            duration : typing___Optional[builtin___int] = None,
            width : typing___Optional[typing___Text] = None,
            style : typing___Optional[typing___Text] = None,
            color : typing___Optional[typing___Text] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"color",b"color",u"duration",b"duration",u"selector",b"selector",u"style",b"style",u"width",b"width"]) -> None: ...
    type___ElementSelectorWithDuration = ElementSelectorWithDuration

    class SelectElementSelector(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        selector: typing___Text = ...
        matcherJson: typing___Text = ...

        def __init__(self,
            *,
            selector : typing___Optional[typing___Text] = None,
            matcherJson : typing___Optional[typing___Text] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"matcherJson",b"matcherJson",u"selector",b"selector"]) -> None: ...
    type___SelectElementSelector = SelectElementSelector

    class JavascriptCode(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        script: typing___Text = ...
        selector: typing___Text = ...

        def __init__(self,
            *,
            script : typing___Optional[typing___Text] = None,
            selector : typing___Optional[typing___Text] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"script",b"script",u"selector",b"selector"]) -> None: ...
    type___JavascriptCode = JavascriptCode

    class WaitForFunctionOptions(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        script: typing___Text = ...
        selector: typing___Text = ...
        options: typing___Text = ...

        def __init__(self,
            *,
            script : typing___Optional[typing___Text] = None,
            selector : typing___Optional[typing___Text] = None,
            options : typing___Optional[typing___Text] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"options",b"options",u"script",b"script",u"selector",b"selector"]) -> None: ...
    type___WaitForFunctionOptions = WaitForFunctionOptions

    class PlaywrightObject(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        info: typing___Text = ...

        def __init__(self,
            *,
            info : typing___Optional[typing___Text] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"info",b"info"]) -> None: ...
    type___PlaywrightObject = PlaywrightObject

    class Viewport(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        width: builtin___int = ...
        height: builtin___int = ...

        def __init__(self,
            *,
            width : typing___Optional[builtin___int] = None,
            height : typing___Optional[builtin___int] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"height",b"height",u"width",b"width"]) -> None: ...
    type___Viewport = Viewport

    class HttpRequest(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        url: typing___Text = ...
        method: typing___Text = ...
        body: typing___Text = ...
        headers: typing___Text = ...

        def __init__(self,
            *,
            url : typing___Optional[typing___Text] = None,
            method : typing___Optional[typing___Text] = None,
            body : typing___Optional[typing___Text] = None,
            headers : typing___Optional[typing___Text] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"body",b"body",u"headers",b"headers",u"method",b"method",u"url",b"url"]) -> None: ...
    type___HttpRequest = HttpRequest

    class HttpCapture(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        urlOrPredicate: typing___Text = ...
        timeout: builtin___float = ...

        def __init__(self,
            *,
            urlOrPredicate : typing___Optional[typing___Text] = None,
            timeout : typing___Optional[builtin___float] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"timeout",b"timeout",u"urlOrPredicate",b"urlOrPredicate"]) -> None: ...
    type___HttpCapture = HttpCapture

    class Device(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        name: typing___Text = ...

        def __init__(self,
            *,
            name : typing___Optional[typing___Text] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"name",b"name"]) -> None: ...
    type___Device = Device

    class AlertAction(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        alertAction: typing___Text = ...
        promptInput: typing___Text = ...

        def __init__(self,
            *,
            alertAction : typing___Optional[typing___Text] = None,
            promptInput : typing___Optional[typing___Text] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"alertAction",b"alertAction",u"promptInput",b"promptInput"]) -> None: ...
    type___AlertAction = AlertAction

    class Bool(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        value: builtin___bool = ...

        def __init__(self,
            *,
            value : typing___Optional[builtin___bool] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...
    type___Bool = Bool


    def __init__(self,
        ) -> None: ...
type___Request = Request

class Types(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    class SelectEntry(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        value: typing___Text = ...
        label: typing___Text = ...
        selected: builtin___bool = ...

        def __init__(self,
            *,
            value : typing___Optional[typing___Text] = None,
            label : typing___Optional[typing___Text] = None,
            selected : typing___Optional[builtin___bool] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"label",b"label",u"selected",b"selected",u"value",b"value"]) -> None: ...
    type___SelectEntry = SelectEntry


    def __init__(self,
        ) -> None: ...
type___Types = Types

class Response(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    class Empty(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        log: typing___Text = ...

        def __init__(self,
            *,
            log : typing___Optional[typing___Text] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"log",b"log"]) -> None: ...
    type___Empty = Empty

    class String(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        log: typing___Text = ...
        body: typing___Text = ...

        def __init__(self,
            *,
            log : typing___Optional[typing___Text] = None,
            body : typing___Optional[typing___Text] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"body",b"body",u"log",b"log"]) -> None: ...
    type___String = String

    class Bool(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        log: typing___Text = ...
        body: builtin___bool = ...

        def __init__(self,
            *,
            log : typing___Optional[typing___Text] = None,
            body : typing___Optional[builtin___bool] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"body",b"body",u"log",b"log"]) -> None: ...
    type___Bool = Bool

    class Int(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        log: typing___Text = ...
        body: builtin___int = ...

        def __init__(self,
            *,
            log : typing___Optional[typing___Text] = None,
            body : typing___Optional[builtin___int] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"body",b"body",u"log",b"log"]) -> None: ...
    type___Int = Int

    class Select(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

        @property
        def entry(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[type___Types.SelectEntry]: ...

        def __init__(self,
            *,
            entry : typing___Optional[typing___Iterable[type___Types.SelectEntry]] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"entry",b"entry"]) -> None: ...
    type___Select = Select

    class Json(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        log: typing___Text = ...
        json: typing___Text = ...

        def __init__(self,
            *,
            log : typing___Optional[typing___Text] = None,
            json : typing___Optional[typing___Text] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"json",b"json",u"log",b"log"]) -> None: ...
    type___Json = Json

    class JavascriptExecutionResult(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        log: typing___Text = ...
        result: typing___Text = ...

        def __init__(self,
            *,
            log : typing___Optional[typing___Text] = None,
            result : typing___Optional[typing___Text] = None,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"log",b"log",u"result",b"result"]) -> None: ...
    type___JavascriptExecutionResult = JavascriptExecutionResult


    def __init__(self,
        ) -> None: ...
type___Response = Response
